package com.ann.demo.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.ann.demo.entity.constant.AliasName;
import com.ann.demo.entity.constant.InterfaceName;
import com.ann.demo.entity.filing.MessageSubordinate;
import com.ann.demo.entity.filing.dto.MessageDto;
import com.ann.demo.entity.filing.dto.PatientMainDto;
import com.ann.demo.entity.filing.ArchiveDetail;
import com.ann.demo.entity.interfaceEntity.ExamApply;
import com.ann.demo.entity.interfaceEntity.MessageLog;
import com.ann.demo.utils.Base64Utils;
import com.ann.demo.utils.FileUtils;
import com.ann.demo.utils.PdfUtils;
import com.ann.demo.utils.XMLUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.ObjectUtils;

import javax.annotation.Resource;
import java.io.File;
import java.util.Date;
import java.util.Objects;
import java.util.UUID;

/**
 * 解析逻辑
 *
 * @Author: LeiJiaXin
 * @Date: 2019/8/20 16:29
 */
@Component
public class AnalysisService {

    static final Logger logger = LoggerFactory.getLogger(AnalysisService.class);

    @Autowired
    MessageService messageService;

    @Autowired
    MessageSubordinateService messageSubordinateService;

    @Autowired
    ExamApplyService examApplyService;

    @Autowired
    ArchiveDetailService archiveDetailService;

    @Resource
    private CollectCheckService collectCheckService;
    /**
     * 2019-7-8
     * @Param xmlStr 需要解析的字符串
     * @Param tClass 被解析字符串的实体类
     * @Param parameter 如果是报告 携带参数
     * 返回返回参数xml
     */
    public String getHIPMessageServerResult(String message, Class tClass, String parameter,String webserviceId) throws Exception{
        String outContent = "失败", typeCodeValue = "AE";
        ArchiveDetail archiveDetail = null;
        //是否走掉接口
        boolean interfaceFlag = false;
        try {
            long startTime = System.currentTimeMillis();
            long endTime = 0;
            String textContent = "成功啦~", type = "", patientMainStr = "", textContentTemp = null, archiveMasterId = null;
            Date xmlCreateTime = null;
            Integer result = 2;
            MessageLog messageLog = new MessageLog(message, InterfaceName.HIP_MESSAGE_SERVER, new Date(), webserviceId);
            messageService.save(messageLog);
            MessageDto messageDto = XMLUtils.xpathToBean(message, tClass, parameter);
            if (messageDto != null) {
                archiveMasterId = messageService.getArchiveMasterId(messageDto.getInpNo(), messageDto.getVisitId());
                if (archiveMasterId != null) {
                    if (Objects.equals(messageDto.getEntity(), ExamApply.class.getName())) { // 申请单
                        ExamApply examApply = JSON.parseObject(messageDto.getObject().toString(), ExamApply.class);
                        textContentTemp = changeApply(examApply, parameter);
                    }
                    //检验外送报告 单独处理
                    if((!ObjectUtils.isEmpty(messageDto.getDeliveryFlag()) )&&
                            Objects.equals(messageDto.getDeliveryFlag(),"广州金域")){
                        // 直接下载并生成文件到detail
                        UUID uuid = UUID.randomUUID();
                        File pdfFile = FileUtils.createFile("pdfs", messageDto.getInpNo(), messageDto.getVisitId(), uuid.toString(), null);
                        if(Base64Utils.isBase64(messageDto.getReportAddress())) {
                            //是base64
                            if (!PdfUtils.base64StringToPDF(messageDto.getReportAddress(), pdfFile)) {
                                textContentTemp = "加密字节流转换pdf失败";
                            }
                        }else{
                            interfaceFlag = true;
                            //不是base64  2023-09-05新增，根据接口下载pdf
                            boolean downFileFlag = collectCheckService.downFileWithCheck(archiveMasterId,messageDto.getBarCode(),messageDto.getCheckReportId(), messageDto.getReportAddress());
                            if(!downFileFlag){
                                textContentTemp = "金域外送base64下载失败";
                            }
                        }
                        if(!interfaceFlag) {
                            // 查询库中是否存在  如果存在就更新
                            archiveDetail = archiveDetailService.isExit(archiveMasterId, messageDto.getType(), messageDto.getSource(), messageDto.getApplyId(), messageDto.getCheckReportId());
                            if (archiveDetail == null) {
                                archiveDetail = new ArchiveDetail(uuid.toString(), archiveMasterId, new Date(), messageDto.getType(), messageDto.getSource(), messageDto.getApplyId(),
                                        messageDto.getDetailType(), messageDto.getCheckReportId());
                                archiveDetail.setPdfPath(pdfFile.getAbsolutePath());
                                archiveDetailService.save(archiveDetail);
                            } else {
                                archiveDetail.setPdfPath(pdfFile.getAbsolutePath());
                                archiveDetailService.save(archiveDetail);
                            }
                        }
                        // 替换消息的pdf节点
                        String tempMessage = XMLUtils.replaceXpath(message, pdfFile.getAbsolutePath());
                        if(!ObjectUtils.isEmpty(tempMessage)){
                            message = tempMessage;
                            messageLog.setInputContent(message);
                            outContent = XMLUtils.generateV3XmlStr("AA", "成功啦~");
                            endTime = System.currentTimeMillis();
                            messageLog.setValue(archiveMasterId, outContent, 1, messageDto == null ? "" : JSON.toJSONString(messageDto), messageDto.getXmlCreateTime(), (endTime - startTime), new Date(), messageDto.getType(), textContent);
                            messageService.save(messageLog);
                            return outContent;
                        }else{
                            textContentTemp = "替换消息的pdf节点出错";
                        }
                    }
                    if (textContentTemp != null) {
                        textContent = textContentTemp;
                    } else {
                        // 构建消息从属表
                        patientMainStr = JSON.toJSONString(new PatientMainDto(messageDto.getInpNo(), messageDto.getVisitId()), SerializerFeature.WriteMapNullValue);
                        xmlCreateTime = messageDto.getXmlCreateTime();
                        type = messageDto.getType();
                        typeCodeValue = "AA";
                        result = 1;
                    }
                } else {
                    textContent = "没有入院信息";
                }
            } else {
                textContent = "传过来的消息无法解析";
            }
            outContent = XMLUtils.generateV3XmlStr(typeCodeValue, textContent);
            if (outContent != null) {
                endTime = System.currentTimeMillis();
                messageLog.setValue(archiveMasterId, outContent, result, messageDto == null ? "" : JSON.toJSONString(messageDto), xmlCreateTime, (endTime - startTime), new Date(), type, textContent);
                messageService.save(messageLog);
                if (Objects.equals(typeCodeValue, "AA") && Objects.equals(type, AliasName.INSPECTION_REPORT)) {
                    messageSubordinateService.save(new MessageSubordinate(messageLog.getId(), patientMainStr, JSON.toJSONString(messageLog)));
                }
            }
        }catch (Exception e){
            e.printStackTrace();
            // 报错删掉患者报告详情信息
            if (archiveDetail != null && !interfaceFlag) {
                if (archiveDetail.getId() != null) {
                    archiveDetailService.delete(archiveDetail.getId());
                }
            }
            // 生成一个临时文件
            logger.error("消息出错，请检查消息内容，错误详情:{},消息所在位置：{}",e.getMessage(),FileUtils.createFile(InterfaceName.HIP_MESSAGE_SERVER,parameter,message));
            return outContent;
        }
        return outContent;
    }

    private String changeApply(ExamApply examApply,String interfaceName)  throws Exception{
        String textContent = null;
        ExamApply examApplyTemp = examApplyService.findExamApply(examApply.getApplyId());
        //1 先判断检验申请单是否存在 存在做以下操作
        if(examApplyTemp != null){
            if (Objects.equals(examApply.getResult(), "delete")) {
                // 1.2 判断是否是删除
                if (!examApplyService.updateExamApplyIsValid(examApplyTemp.getApplyId())) {
                    textContent = "申请单删除失败了";
                }
            }else if(Objects.equals(interfaceName,InterfaceName.PARAMETER_INSPECTIONAPPLY)){
                // 1.3 如果是检验申请单状态变更设置IsOpenHIS
                if (Objects.equals(examApply.getIsOpenHis(), "F")) {
                    examApplyTemp.setIsValid(1);
                    examApplyTemp.setIsOpenHis("F");
                    examApplyService.save(examApplyTemp);
                }
            }else if(Objects.equals(interfaceName,InterfaceName.PARAMETER_VERIFICATIONINSPECTIONAPPLY)){
                // 1.4 如果是检验申请单检体验收设置IsOpenLIS
                if (Objects.equals(examApply.getIsOpenOther(), "F")) {
                    examApplyTemp.setIsValid(1);
                    examApplyTemp.setIsOpenOther("F");
                    examApplyService.save(examApplyTemp);
                }
            }else {
                // 1.5 如果存在，又不做任何操作，返回已存在
                textContent = "该申请单已存在";
            }
        }else{
            // 2 如果不存在，那就判断是否是新增，否则该申请单不存在
            if (Objects.equals(examApply.getResult(), "new") && Objects.equals(interfaceName,InterfaceName.PARAMETER_INSPECTIONAPPLY)) {
                if (examApply.getReportType() != null) {
                    examApply.setReportType(AliasName.INSPECTION_REPORT);
                }
                examApplyService.save(examApply);
            } else {
                textContent = "该申请单不存在";
            }
        }
        return textContent;
    }


}
